/*
libfawk - A function-only AWK dialect

Copyright (c) 2017..2020 Tibor 'Igor2' Palinkas. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

  * 1. Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
  * 3. Neither the name of the copyright holder nor the names of its
    contributors may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Project page: http://repo.hu/projects/libfawk
Source code: svn://repo.hu/libfawk/trunk
Contact the author: http://igor2.repo.hu/contact.html
*/

#include "libfawk.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "str.h"
#include "execute.h"
#include "parser.h"

FAWK_API int fawk_cast_to_num(fawk_ctx_t *ctx, fawk_cell_t *cell)
{
	fawk_num_t res;

	switch(cell->type) {
		case FAWK_NUM: return 0;
		case FAWK_STRNUM:
			res = cell->data.str->num;
			break;
		case FAWK_STR:
			res = fawk_strtod(cell->data.str->str);
			break;
		case FAWK_FUNC:
			res = cell->data.func.ip;
			break;
		case FAWK_ARRAY:
			res = cell->data.arr->uid;
			break;
		case FAWK_SYMREF:
		case FAWK_CCALL_RET:
			FAWK_ERROR(ctx, 32, (FAWK_ERR, "cast-to-num: invalid type\n"));
			return -1;
		case FAWK_NIL:
			res = 0;
			break;
	}

	fawk_cell_free(ctx, cell);
	cell->type = FAWK_NUM;
	cell->data.num = res;

	return 0;
}

FAWK_API int fawk_cast_to_str(fawk_ctx_t *ctx, fawk_cell_t *cell)
{
	char buff[128];
	const char *res;
	fawk_num_t n;

	switch(cell->type) {
		case FAWK_NUM:
			sprintf(buff, FAWK_NUM_PRINTF_FMT, cell->data.num);
			n = cell->data.num;
			cell->data.str = fawk_str_new_from_literal(ctx, buff, -1);
			cell->data.str->num = n;
			cell->type = (cell->data.str == NULL) ? FAWK_NIL : FAWK_STRNUM; /* special case: we know the numeric value */
			return 0;
		case FAWK_STRNUM:
		case FAWK_STR:
			return 0;
		case FAWK_FUNC:
			res = cell->data.func.name;
			break;
		case FAWK_SYMREF:
		case FAWK_ARRAY:
		case FAWK_CCALL_RET:
			FAWK_ERROR(ctx, 32, (FAWK_ERR, "cast-to-str: invalid type\n"));
			return -1;
		case FAWK_NIL:
			res = "";
			break;
	}

	/* no need to free cell as it was scalar */
	cell->data.str = fawk_str_new_from_literal(ctx, res, -1);
	cell->type = (cell->data.str == NULL) ? FAWK_NIL : FAWK_STR;
	return 0;
}
